"use server";

import { nextauthOptions } from "@/app/lib/nextauth-options";
import { getServerSession } from "next-auth";


const getGlobalHeaders = async () => {
  const session = await getServerSession(nextauthOptions);

  return {
    Authorization: `Bearer ${session?.user?.token}`,
    "Content-Type": "application/json",
    Accept: "application/json",
    "x-project-id": session?.currentProjectId ?? "",
  };
};


interface getFetch {
  path: string;
  options?: RequestInit;
}

export const requestGetFetch = async ({ path, options = {} }: getFetch) => {
  try {
    const headers = await getGlobalHeaders();
    const finalHeader = {
      ... options.headers,
      ...headers
    }
    options.headers = finalHeader;
    const requestOptions: RequestInit = {
      method: "GET",
      headers: {
        "Content-Type": "application/json",
      },
      ...options,
    };

    const url = process.env.API_BASE_URL + path;
    const data = await fetch(url, requestOptions);

    const result = await data.json();
    
    return { ...result, status: data.status };

  } catch (err) {
    if (err instanceof Error) {
      throw new Error(err.message);
    }
    throw err;
  }
};

interface postFetch {
  path: string;
  options?: RequestInit;
  token: string;
  data: FormData | any;
}

export const requestPostFetch = async ({
  path,
  data,
  token,
  options = {},
}: postFetch): Promise<any> => {
  let requestOptions: RequestInit;
  const headers = await getGlobalHeaders();
  const finalHeader = {
    ...options.headers,
    ...headers,
  };

  if (data instanceof FormData) {

    delete finalHeader["Content-Type"];
    delete finalHeader["Accept"];
    options.headers = finalHeader;

    requestOptions = {
      method: "POST",
      headers: {
        Authorization: `Bearer ${token}`,
      },
      body: data,
      ...options,
    };
  } else {
    options.headers = finalHeader;

    requestOptions = {
      method: "POST",
      headers: {
        "Content-Type": "application/json",
        Authorization: `Bearer ${token}`,
      },
      body: JSON.stringify(data || {}),
      ...options,
    };
  }

  const url = process.env.API_BASE_URL + path;
  const response = await fetch(url, requestOptions);
  const result = await response.json();


  if (!response.ok) {
    throw new Error(result.message || "An error occurred");
  }
  return result;
};

